@@ -107,6 +107,14 @@ Example (in C<debian/control>):
                     labix.org/v2/mgo,
                     launchpad.net/mgo
 
+C<DH_GOPKG> is set by dh-golang, and as a consequence it is not present in the
+C<debian/rules> environment. If you need to use the Go package name in the
+C<debian/rules> file, you must define it yourself.
+
+Example (in C<debian/rules>):
+
+ export DH_GOPKG := github.com/go-mgo/mgo
+
 Historical note: before the C<XS-Go-Import-Path> field was introduced, we used
 to set C<DH_GOPKG> in C<debian/rules>. When you encounter such a package, please
 convert it by moving the value from C<debian/rules> to C<debian/control>. It is
@@ -119,8 +127,8 @@ very hard to parse.
 C<DH_GOLANG_INSTALL_EXTRA> (list of strings, whitespace-separated, default
 empty) enumerates files and directories which are additionally installed into
 the build directory. By default, only files with the following extension are
-installed: .go, .c, .cc, .h, .hh, .proto, .s. Starting with dh-golang 1.31,
-testdata directory contents are installed by default.
+installed: .go, .c, .cc, .cpp, .h, .hh, hpp, .proto, .s. Starting with dh-golang
+1.31, testdata directory contents are installed by default.
 
 Example (in C<debian/rules>):
 
@@ -130,8 +138,8 @@ Example (in C<debian/rules>):
 
 C<DH_GOLANG_INSTALL_ALL> (bool, default false) controls whether all files are
 installed into the build directory. By default, only files with the following
-extension are installed: .go, .c, .cc, .h, .hh, .proto, .s. Starting with
-dh-golang 1.31, testdata directory contents are installed by default.
+extension are installed: .go, .c, .cc, .cpp, .h, .hh, .hpp, .proto, .s. Starting
+with dh-golang 1.31, testdata directory contents are installed by default.
 
 Example (in C<debian/rules>):
 
@@ -165,12 +173,31 @@ C<DH_GOLANG_EXCLUDES> (list of Perl regular expressions, whitespace-separated,
 default empty) defines regular expression patterns to exclude from the build
 targets expanded from C<DH_GOLANG_BUILDPKG>.
 
+Please note that with DH_COMPAT level inferior or equal to 11, the default is
+to only exclude pattern from the build target.  (see C<DH_GOLANG_EXCLUDES_ALL>
+below)
+
 Example (in C<debian/rules>):
 
- # We want to ship only the library packages themselves, not the accompanying
+ # We want to build only the library packages themselves, not the accompanying
  # example binaries.
  export DH_GOLANG_EXCLUDES := examples/
 
+=item DH_GOLANG_EXCLUDES_ALL
+
+C<DH_GOLANG_EXCLUDES_ALL> (boolean, default to true starting from DH_COMPAT
+level 12) makes C<DH_GOLANG_EXCLUDE> excludes files not only during the
+building process but also for install.  This is useful, if, for instance,
+examples are installed with C<dh_installexamples>. If you only want to
+exclude files from the building process but keep them in the source, set this
+to false.
+Example (in C<debian/rules>):
+
+ # We want to ship only the library packages themselves in the go source, not
+ # the accompanying example binaries.
+ export DH_GOLANG_EXCLUDES := examples/
+ export DH_GOLANG_EXCLUDES_ALL := 1
+
 =item DH_GOLANG_GO_GENERATE
 
 C<DH_GOLANG_GO_GENERATE> (bool, default false) controls whether C<go generate>
@@ -216,7 +243,7 @@ use strict;
 use base 'Debian::Debhelper::Buildsystem';
 use Debian::Debhelper::Dh_Lib;
 use Dpkg::Control::Info;
-use File::Copy; # in core since 5.002
+use File::Copy "cp"; # in core since 5.002
 use File::Path qw(make_path); # in core since 5.001
 use File::Find; # in core since 5
 use File::Spec; # in core since 5.00405
@@ -313,8 +340,10 @@ sub configure {
         '.go' => 1,
         '.c' => 1,
         '.cc' => 1,
+        '.cpp' => 1,
         '.h' => 1,
         '.hh' => 1,
+        '.hpp' => 1,
         '.proto' => 1,
         '.s' => 1,
     );
@@ -384,7 +413,7 @@ sub configure {
 
         make_path(dirname($dest));
         verbose_print("Copy $source -> $dest");
-        copy($source, $dest) or error("Could not copy $source to $dest: $!");
+        cp($source, $dest) or error("Could not copy $source to $dest: $!");
     }
 
     ############################################################################
@@ -483,8 +512,50 @@ sub install {
     if ($install_source) {
         # Path to the src/ directory within $destdir
         my $dest_src = "$destdir/usr/share/gocode/src/$ENV{DH_GOPKG}";
-        $this->doit_in_builddir('mkdir', '-p', $dest_src);
-        $this->doit_in_builddir('cp', '-r', '-T', "src/$ENV{DH_GOPKG}", $dest_src);
+
+        # starting from compat level 12, exclude_all defaults to True
+        my $exclude_all_default = (compat(11) ?
+                                0 : 1);
+
+        my $exclude_all = (exists($ENV{DH_GOLANG_EXCLUDES_ALL}) ?
+                            $ENV{DH_GOLANG_EXCLUDES_ALL} : $exclude_all_default);
+
+        my @excludes = (exists($ENV{DH_GOLANG_EXCLUDES}) && $exclude_all ?
+                        split(/ /, $ENV{DH_GOLANG_EXCLUDES}) : ());
+
+        find({
+            wanted => sub {
+                my $source = $File::Find::name;
+                my $source_rel = File::Spec-> abs2rel($source, "$builddir/src/$ENV{DH_GOPKG}");
+
+                for my $pattern (@excludes) {
+                    if ($source_rel =~ /$pattern/) {
+                        verbose_print("$source_rel matches $pattern from DH_GOLANG_EXCLUDES, skipping\n");
+                        return;
+                    }
+                }
+
+                my $dest = "$dest_src/$source_rel";
+                my $destdir = dirname($dest);
+
+                return if (-e $dest);
+                return if (-d $source);
+                make_path($destdir) unless (-d $destdir);
+
+                # it's very unlikely there are symlinks. But just in case...
+                if (-l $source) {
+                    my $link_target = readlink($source);
+                    verbose_print("Create symlink $dest -> $link_target");
+                    symlink($link_target, $dest) or error("Could not create symlink $dest -> $link_target: $!");
+                    return
+                }
+
+                verbose_print("Copy $source -> $dest");
+                cp($source, $dest) or error("Could not copy $source to $dest: $!");
+
+                },
+            no_chdir => 1,
+            }, "$builddir/src/$ENV{DH_GOPKG}");
     }
 }
 
